package RSSReader::Category;
use strict;
use RSSReader::Configuration;
use RSSReader::Feed;
use Data::Dumper;
use base qw(RSSReader::Notifier RSSReader::Logger);

our $_cats;

sub new {
	my $c = shift;
	my %a = @_;

    	if (exists($_cats->{$a{id}}->{cat})) { return $_cats->{$a{id}}->{cat} }
    	$c->log_debug("Category $a{id} was not cached");
	
	my $config   = RSSReader::Configuration->new();	
	my $cat_info = $config->get_category(id => $a{id});
	$_cats->{$a{id}}->{cat} = bless { info => $cat_info, id=>$a{id} }, $c;
			
	return $_cats->{$a{id}}->{cat};
}

sub create {
        my $c = shift;
        my %a = @_;

        die "I need a location " unless $a{location};
        die "I need a label" unless exists ($a{label});

        my $result = RSSReader::Configuration->new()->insert_category(%a, location=>$a{location}->id());
        return undef unless $result;
        $c->log_debug("Created category $result");
        my $s = __PACKAGE__->new(id=>$result);
        return undef unless $s;
        $a{location}->_new_category( category=> $s) if $a{location}; 
        return $s;
}

sub delete {
	my $s = shift;

	my @feeds = $s->get_feeds();
	my @cats  = $s->get_sub_categories();

	foreach ( @cats, @feeds) {
		$_->delete();
	}
	$s->notify('category_deleted');
	RSSReader::Configuration->new()->remove_category(id=>$s->id);
	return 1;
}

sub label {
	my $s = shift;

	return $s->{info}->{label};	
}

sub location {
	my $s = shift;
	if ($_[0]) { 
	   $s->{info}->{location}=$_[0];
	}
	return $s->{info}->{location};	

}

sub set_label {
	my $s = shift;
	my %a = @_;

	$s->{info}->{label} = $a{new_label};
	$s->notify('label_change');
}

sub id {
	my $s = shift;
	return $s->{id};	
}

sub get_sub_categories {
	my $s = shift;
	my %a = @_;
	
	return $s->search(location=>$s->id());
}

sub has_sub_categories {
	my $s = shift;
	
	return 1 if scalar(@{$s->get_sub_categories});
	
	return 0;
}

sub move {
	my $s = shift;
	my %a = @_;

	my $from = __PACKAGE__->new(id=>$s->location);

	$s->log_debug("I was asked to move to ".$a{to}->label());
	if ($s->is_my_sub_category($a{to}, recursive=>1)) {
		$@ = "cannot move to sub category";
		print STDERR $@."\n";
		return undef;
	}

	$s->notify('category_moved', to=> $a{to}, from=>$from);
	RSSReader->new->notify('category_moved', to=> $a{to}, from=>$from, category=>$s);
	$s->notify('category_moved_before', to=> $a{to}, from=>$from);
	RSSReader->new->notify('category_moved_before', to=> $a{to}, from=>$from, category=>$s);
	$s->location($a{to}->id());
	$s->notify('category_moved_after',  to=> $a{to}, from=>$from);
	RSSReader->new->notify('category_moved_after',  to=> $a{to}, from=>$from, category=>$s);
	return 1;
}

sub messages {
	my $s = shift;
	my @m;
	foreach my $child (@{$s->get_feeds()}, @{$s->get_sub_categories}) {
	        push @m, @{$child->messages};
	}
	return wantarray ? @m : \@m;
}

sub unread_messages {
	my $s = shift;

	my @unread;
	
	foreach my $child (@{$s->get_feeds()}, @{$s->get_sub_categories}) {
		push @unread, @{$child->unread_messages};
	}

	return wantarray ? @unread : \@unread;
	
}

sub mark_all_read {
	my $s = shift;
	foreach my $m ( @{$s->unread_messages} ) {
		$m->mark_read;
	}
	return 1;
}

sub mark_all_unread {
	my $s = shift;
	foreach my $m ( @{$s->messages} ) { $m->mark_unread }
	return 1;
}

sub has_feeds {	
	my $s = shift;
	
	return scalar( @{RSSReader::Feed->search_feeds(location=>$s->id())} );
}

sub get_feeds {
	my $s = shift;
	
	return wantarray ? @{RSSReader::Feed->search_feeds(location=>$s->id())} : RSSReader::Feed->search_feeds(location=>$s->id());
}

sub is_my_sub_category {
	my $s = shift;
	my $cat = shift;
	my %a = @_;

	foreach my $c (@{$s->get_sub_categories}) {
		return $c if ($cat->id == $c->id);
	}

	if ($a{recursive}) {
		foreach my $c (@{$s->get_sub_categories}) {
			return $c if $c->is_my_sub_category($cat, %a);
		}
	}
}

sub is_my_feed {
	my $s = shift;
	my $feed = shift;
	my %a = @_;

	foreach my $f (@{$s->get_feeds}) {
		return $feed if ($feed->id == $f->id);
	}

	if ($a{recursive}) {
		foreach my $c (@{$s->get_sub_categories}) {
			return $feed if $c->is_my_feed($feed, %a);
		}
	}

	return 0;
}

sub search {
        my $s = shift;
        my %a = @_;

        my @cats_to_return = ();

        my $conf = RSSReader::Configuration->new();
        my @ids  = $conf->search_categories();

        foreach my $id (@ids) {
                my $cat = __PACKAGE__->new(id=>$id);
                if ($a{location} && $cat->location()) {
                        push @cats_to_return, $cat if $cat->location() == $a{location};
                }
        }

        return wantarray ? @cats_to_return : \@cats_to_return;
}


sub _new_feed {
	my $s = shift;
	my %a = @_;
	$s->log_debug("I got a new feed ".$a{feed}->label());
	$s->notify('new_feed', feed=>$a{feed});
}

sub _new_category {
	my $s = shift;
	my %a = @_;
	$s->log_debug("I got a new category ".$a{category}->label());
	$s->notify('new_category', category=>$a{category});
}

##### MESSAGE NOTIFICATIONS
sub _feed_deleted {
	my $s = shift;
	my %a = @_;
	$s->notify('feed_deleted', feed => $a{where});

}
sub _feed_deleted_after {
	my $s = shift;
	my %a = @_;
	$s->notify('feed_deleted_after', feed => $a{where});
}

sub _message_marked_read {
	my $s = shift;
	my %a = @_;

	RSSReader::Category->new(id=>$s->location())->_message_marked_read( message => $a{message} ) if $s->location;
	$s->notify('message_marked_read', message => $a{message});
}

sub _message_marked_unread {
	my $s = shift;
	my %a = @_;

	RSSReader::Category->new(id=>$s->location)->_message_marked_unread( message => $a{message} ) if $s->location;
	$s->notify('message_marked_unread', message => $a{message});
}

sub _new_message {
	my $s = shift;
	my %a = @_;

	RSSReader::Category->new(id=>$s->location)->_new_message( message => $a{message} ) if $s->location;
	$s->notify('new_message', message => $a{message});
}

sub _message_deleted {
        my $s = shift;
        my %a = @_;
	
	RSSReader::Category->new(id=>$s->location)->_message_deleted( message => $a{message} ) if $s->location;
        $s->notify('message_deleted', message => $a{message});
}
1;
